@@ -1,6 +1,7 @@
 import numpy as np
 import pandas as pd
 
+from linora.metrics._utils import _sample_weight
 from linora.metrics._regression import mean_squared_error
 
 __all__ = ['accuracy_binary', 'accuracy_categorical', 'recall', 'precision', 'confusion_matrix',
@@ -8,7 +9,7 @@ __all__ = ['accuracy_binary', 'accuracy_categorical', 'recall', 'precision', 'co
            'crossentropy_categorical', 'ks', 'gini', 'psi', 'fmi', 'report_binary',
            'accuracy_categorical_top_k', 'iou_binary', 'iou_categorical',
            'precision_on_recall', 'recall_on_precision', 
-           'specificity_on_sensitivity', 'sensitivity_on_specificity'
+           'specificity_on_sensitivity', 'sensitivity_on_specificity', 'best_prob', 'matthews_score'
           ]
 
 
@@ -23,16 +24,6 @@ def classified_func(y_true, y_pred, prob=0.5, pos_label=1):
     return t
 
 
-def _sample_weight(y_true, sample_weight):
-    if sample_weight is None:
-        sample_weight = np.ones(len(y_true))
-    elif isinstance(sample_weight, dict):
-        sample_weight = [sample_weight[i] for i in y_true]
-    else:
-        sample_weight = np.array(sample_weight)
-    return sample_weight/sample_weight.sum()*len(sample_weight)
-
-
 def accuracy_binary(y_true, y_pred, sample_weight=None, prob=0.5, pos_label=1):
     """Calculates how often predictions match binary labels.
     
@@ -70,6 +61,37 @@ def accuracy_categorical(y_true, y_pred, sample_weight=None):
     return ((pd.Series(y_true)==pd.Series(y_pred))*sample_weight).mean()
 
 
+def matthews_score(y_true, y_pred, sample_weight=None, prob=0.5, pos_label=1):
+    """Compute the Matthews correlation coefficient (MCC).
+
+    The Matthews correlation coefficient is used in machine learning as a measure of 
+    the quality of binary and multiclass classifications. 
+    It takes into account true and false positives and negatives and is generally 
+    regarded as a balanced measure which can be used even if the classes are of very different sizes. 
+    The MCC is in essence a correlation coefficient value between -1 and +1. 
+    A coefficient of +1 represents a perfect prediction, 
+    0 an average random prediction and -1 an inverse prediction. 
+    The statistic is also known as the phi coefficient.
+    
+    Args:
+        y_true: pd.Series or array or list, ground truth (correct) labels.
+        y_pred: pd.Series or array or list, predicted labels, as returned by a classifier.
+        sample_weight: list or array or dict of sample weight.
+        prob: probability threshold.
+        pos_label: positive label.
+    Returns:
+        The Matthews correlation coefficient (+1 represents a perfect prediction, 0 an average random prediction and -1 and inverse prediction).
+    """
+    sample_weight = _sample_weight(y_true, sample_weight)
+    t = classified_func(y_true, y_pred, prob=prob, pos_label=pos_label)
+    t['weight'] = sample_weight
+    tp = (((t.label==pos_label)&(t.prob==pos_label))*t.weight).sum()
+    fp = (((t.label==pos_label)&(t.prob!=pos_label))*t.weight).sum()
+    fn = (((t.label!=pos_label)&(t.prob==pos_label))*t.weight).sum()
+    tn = (((t.label!=pos_label)&(t.prob!=pos_label))*t.weight).sum()
+    return (tp*tn-fp*fn)/np.sqrt((tp+fp)*(tp+fn)*(tn+fp)*(tn+fn))
+
+
 def iou_binary(y_true, y_pred, sample_weight=None, prob=0.5, pos_label=1):
     """Computes the Intersection-Over-Union metric for class 0 and/or 1.
     
@@ -169,7 +191,7 @@ def _cumsum_confusion_matrix(y_true, y_pred, sample_weight=None, pos_label=1):
     t['tp'] = (t.label*t.weight).cumsum()
     t['fp'] = t.weight.cumsum()-t.tp
     t['tn'] = ((1-t.label)*t.weight).iloc[::-1].cumsum()
-    t['fn'] = t.weight.iloc[::-1].cumsum()-t.tn
+    t['fn'] = t.tp.max()-t.tp#t.weight.iloc[::-1].cumsum()-t.tn
     t['recall'] = t.tp/(t.tp + t.fn)
     t['precision'] = t.tp/(t.tp+t.fp)
     t['specificity'] = t.tn/(t.tn + t.fp)
@@ -177,6 +199,44 @@ def _cumsum_confusion_matrix(y_true, y_pred, sample_weight=None, pos_label=1):
     return t
 
 
+def best_prob(y_true, y_pred, precision=None, recall=None, specificity=None, 
+              accuracy=None, f1_score=None, iou_binary=None,
+              sample_weight=None, pos_label=1):
+    """Computes best best prob threshold when method is >= specified value.
+    
+    Args:
+        y_true: pd.Series or array or list, ground truth (correct) labels.
+        y_pred: pd.Series or array or list, predicted labels, as returned by a classifier.
+        precision: A scalar value in range [0, 1].
+        recall: A scalar value in range [0, 1].
+        specificity: A scalar value in range [0, 1].
+        accuracy: A scalar value in range [0, 1].
+        f1_score: A scalar value in range [0, 1].
+        iou_binary: A scalar value in range [0, 1].
+        sample_weight: list or array or dict of sample weight.
+        pos_label: positive label.
+    Returns:
+        best prob threshold.
+    """
+    t = _cumsum_confusion_matrix(y_true, y_pred, sample_weight=sample_weight, pos_label=pos_label)
+    t['accuracy'] = (t.tp+t.tn)/(t.tp + t.fn+t.tn+t.fp)
+    t['f1_score'] = 2*t.precision*t.recall/(t.precision+t.recall)
+    t['iou_binary'] = t.tp/(t.tp+t.fp+t.fn)
+    if recall is not None:
+        t = t[t.recall>=recall]
+    if precision is not None:
+        t = t[t.precision>=precision]
+    if specificity is not None:
+        t = t[t.specificity>=specificity]
+    if accuracy is not None:
+        t = t[t.accuracy>=accuracy]
+    if f1_score is not None:
+        t = t[t.f1_score>=f1_score]
+    if iou_binary is not None:
+        t = t[t.iou_binary>=iou_binary]
+    return t.prob.min()
+
+
 def precision_on_recall(y_true, y_pred, recall=0.5, sample_weight=None, pos_label=1):
     """Computes best precision where recall is >= specified value.
     
@@ -210,7 +270,7 @@ def recall_on_precision(y_true, y_pred, precision=0.5, sample_weight=None, pos_l
 
 
 def sensitivity_on_specificity(y_true, y_pred, specificity=0.5, sample_weight=None, pos_label=1):
-    """Computes best sensitivity where specificity is >= specified value.
+    """Computes best sensitivity(recall) where specificity is >= specified value.
     
     Args:
         y_true: pd.Series or array or list, ground truth (correct) labels.
@@ -226,7 +286,7 @@ def sensitivity_on_specificity(y_true, y_pred, specificity=0.5, sample_weight=No
 
 
 def specificity_on_sensitivity(y_true, y_pred, sensitivity=0.5, sample_weight=None, pos_label=1):
-    """Computes best specificity where sensitivity is >= specified value.
+    """Computes best specificity where sensitivity(recall) is >= specified value.
     
     Args:
         y_true: pd.Series or array or list, ground truth (correct) labels.
@@ -299,12 +359,20 @@ def auc_roc(y_true, y_pred, sample_weight=None, pos_label=1):
     Returns:
         Area Under the Receiver Operating Characteristic Curve (ROC AUC) from prediction scores.
     """
+#     sample_weight = _sample_weight(y_true, sample_weight)
+#     t = pd.DataFrame({'prob':y_pred, 'label':y_true, 'weight':sample_weight})
+#     assert t.label.nunique()==2, "`y_true` should be binary classification."
+#     t.insert(0, 'target', t[t.label!=pos_label].label.unique()[0])
+#     t = t[t.label!=pos_label].merge(t[t.label==pos_label], on='target')
+#     auc = ((t.prob_y>t.prob_x)*(t.weight_y+t.weight_x)/2).mean()+((t.prob_y==t.prob_x)*(t.weight_y+t.weight_x)/2).mean()/2
+    
     sample_weight = _sample_weight(y_true, sample_weight)
-    t = pd.DataFrame({'prob':y_pred, 'label':y_true, 'weight':sample_weight})
+    t = pd.DataFrame({'prob':y_pred, 'label':y_true, 'weight':sample_weight}).sort_values(['prob']).reset_index(drop=True)
     assert t.label.nunique()==2, "`y_true` should be binary classification."
-    t.insert(0, 'target', t[t.label!=pos_label].label.unique()[0])
-    t = t[t.label!=pos_label].merge(t[t.label==pos_label], on='target')
-    auc = ((t.prob_y>t.prob_x)*(t.weight_y+t.weight_x)/2).mean()+((t.prob_y==t.prob_x)*(t.weight_y+t.weight_x)/2).mean()/2
+    pos_rank = ((t[t.label==pos_label].index+1).values*(t[t.label==pos_label].weight.values)).sum()
+    pos_cnt = t[t.label==pos_label].weight.sum()
+    neg_cnt = t[t.label!=pos_label].weight.sum()
+    auc = (pos_rank - pos_cnt*(pos_cnt+1)/2) / (pos_cnt*neg_cnt)
     return auc
 
 
